@@ -23,7 +23,7 @@ import itertools
 import collections
 
 __author__  = 'Miguel Turner'
-__version__ = '0.3.3'
+__version__ = '0.4.0'
 __license__ = 'MIT'
 
 __all__ = ['Config', 'IniFormat', 'ConfigError', 'Coercer', 'CoerceError']
@@ -135,9 +135,7 @@ class ConfigSection(collections.MutableMapping):
         the sources.
         """
         format = kwargs.pop('format', None)
-        if kwargs:
-            err = "sync() got an unexpected keyword argument '{}'"
-            raise TypeError(err.format(kwargs.popitem()[0]))
+        kwargs_check('sync', kwargs)
         
         sources, format = self._process_sources(sources, format)
         
@@ -154,6 +152,8 @@ class ConfigSection(collections.MutableMapping):
         *sources* can be set using *format*.
         """
         format = kwargs.pop('format', None)
+        kwargs_check('read', kwargs)
+        
         sources, format = self._process_sources(sources, format)
         self._read(sources, format)
     
@@ -240,7 +240,7 @@ class ConfigSection(collections.MutableMapping):
         return "{}('{}', value={!r}, keys={}, comment={!r})".format(
             self.__class__.__name__, self.key, value, list(self), self.comment)
     
-    def as_dict(self, flat=False, recurse=True, dict_type=None):
+    def as_dict(self, flat=False, dict_type=None):
         """Returns the configuration's keys and values as a dictionary.
         
         If *flat* is `True`, returns a single-depth dict with :samp:`.`
@@ -257,18 +257,16 @@ class ConfigSection(collections.MutableMapping):
             sections = ((k, self.section(k)) for k in self)
             return dtype((k, s.value()) for k, s in sections)
         
-        if recurse and self._children:
-            d = dtype()
-            if valid:
-                d[''] = self.value()
-            for section in self.sections():
-                d.update(section.as_dict(dict_type=dict_type))
-            
-            return d if self is self._root else dtype({self.name: d})
-        elif valid:
-            return dtype({self.name: self.value()})
-        else:
-            return dtype()
+        d = dtype()
+        if valid:
+            d[''] = self.value()
+        for section in self.sections():
+            if section._children:
+                d[section.name] = section.as_dict(dict_type=dict_type)
+            else:
+                d[section.name] = section.value()
+        
+        return d
 
     def section(self, key, create=True):
         """Returns a section object for *key*.
@@ -284,6 +282,9 @@ class ConfigSection(collections.MutableMapping):
                 section = section._children[name]
             except KeyError:
                 if create:
+                    if self._root.strict:
+                        err = 'cannot create a section for an uninitialized key: {}'
+                        raise StrictError(err.format(key))
                     return self._create_section(key)
                 raise InvalidSectionError(key)
         return section
@@ -323,6 +324,9 @@ class ConfigSection(collections.MutableMapping):
     
     def set_value(self, value):
         """Set the section's value."""
+        if self._root.strict and self._default is NoValue:
+            err = 'cannot set a value for an uninitialized key: {}'
+            raise StrictError(err.format(self.key))
         self._value = value
         self._dirty = True
     
@@ -336,19 +340,22 @@ class ConfigSection(collections.MutableMapping):
         """Set the section's default value."""
         self._default = value
     
-    def adapt(self):
+    def adapt(self, encode=True):
         """value -> str"""
         if not self._root.coercer:
             return value
-        return self._root.coercer.adapt(self.value(), self._type)
+        value = self._root.coercer.adapt(self.value(), self._type)
+        if encode and isinstance(value, str):
+            value = value.encode(self._root.encoding)
+        return value
     
-    def convert(self, string):
+    def convert(self, string, decode=True):
         """str -> value"""
         if self._root.coercer:
             type = self._type
             # if we are converting a byte-string and the type is not bytes,
             # then we need to decode it
-            if isinstance(string, bytes) and not (
+            if decode and isinstance(string, bytes) and not (
                 inspect.isclass(type) and issubclass(type, bytes)
                 ):
                 string = string.decode(self._root.encoding)
@@ -369,14 +376,14 @@ class ConfigSection(collections.MutableMapping):
             try:
                 file = format.open(source)
             except IOError as e:
-                log.debug('%s: %s', source, e)
+                log.warning('%s: %s', source, e)
                 continue
             
             # read file
             try:
                 lines = format.read(file)
             except IOError as e:
-                log.debug('%s: %s', source, e)
+                log.warning('%s: %s', source, e)
                 continue
             finally:
                 # only close files that were opened from the filesystem
@@ -390,7 +397,7 @@ class ConfigSection(collections.MutableMapping):
                 write_lines = lines
         
         if not one_source_read:
-            log.info('no sources were read')
+            log.warning('no config was read')
         
         return write_lines
     
@@ -511,17 +518,25 @@ class Config(ConfigSection):
 
         self.sources = list(sources)
         self.encoding = kwargs.pop('encoding', locale.getpreferredencoding(False))
+        self.strict = kwargs.pop('strict', False)
         
         format = kwargs.pop('format', 'ini')
         self.set_format(format)
         
         self.coercer = kwargs.pop('coercer', NoValue)
         if self.coercer is NoValue:
-            self.coercer = Coercer(self.encoding)
+            self.coercer = Coercer()
         if self.coercer:
             register_booleans(self.coercer)
         
         self.sep = '.'
+        
+        kwargs_check('__init__', kwargs)
+    
+    @property
+    def format(self):
+        """The :class:`~profig.Format` to use to process sources."""
+        return self._format
     
     @classmethod
     def known_formats(cls):
@@ -564,14 +579,25 @@ class Line(collections.namedtuple('Line', 'line name iskey issection')):
 
 class Format(BaseFormat):
     name = None
+    error_modes = {'ignore', 'warning', 'exception'}
     
     def __init__(self, config):
+        #: Access to the root Config instance.
         self.config = config
         
         self.encoding = config.root.encoding
         self.ensure_dirs = 0o744
-        self.read_errors = 'exception'
-        self.write_errors = 'exception'
+        self.error_mode = 'warning'
+    
+    @property
+    def error_mode(self):
+        return self._error_mode
+    
+    @error_mode.setter
+    def error_mode(self, mode):
+        if mode not in self.error_modes:
+            raise ValueError('invalid error_mode: {}'.format(mode))
+        self._error_mode = mode
     
     def read(self, file): # pragma: no cover
         """Reads *file* and returns a dict. Must be implemented
@@ -611,27 +637,26 @@ class Format(BaseFormat):
     def flush(self, file):
         file.flush()
     
-    def _read_error(self, file, lineno=None, text='', message=''):
-        if self.read_errors != 'ignore':
-            name = file.name if hasattr(file, 'name') else '<???>'
-            exc = ReadError(name, lineno, text, message)
-            if self.read_errors == 'exception':
-                raise exc
-            elif self.read_errors == 'error':
-                print(str(exc), file=sys.stderr)
-            else:
-                assert False
-    
-    def _write_error(self, file, message=''):
-        if self.write_errors != 'ignore':
-            name = file.name if hasattr(file, 'name') else '<???>'
-            exc = WriteError(name, message)
-            if self.write_errors == 'exception':
-                raise exc
-            elif self.write_errors == 'error':
-                print(str(exc), file=sys.stderr)
-            else:
-                assert False
+    def _error(self, exc, file, lineno=None, text=''):
+        if self.error_mode == 'ignore':
+            return
+        
+        name = file.name if hasattr(file, 'name') else '<???>'
+        err = ["error reading '{}'".format(name)]
+        if lineno is not None:
+            err.append(', line {}'.format(lineno))
+        err.append(': {}'.format(exc))
+        if text:
+            err.append('\n  {}'.format(text))
+        message = ''.join(err)
+        
+        if self.error_mode == 'exception':
+            log.error(message)
+            raise exc
+        elif self.error_mode == 'warning':
+            log.warning(message)
+        else:
+            assert False
 
 class IniFormat(Format):
     name = 'ini'
@@ -690,7 +715,7 @@ class IniFormat(Format):
             try:
                 key, value = line.split(self.delimeter.strip(), 1)
             except ValueError:
-                self._read_error(file, i, line)
+                self._error(FormatError('invalid syntax'), file, i, line)
                 continue
             
             key = cfg._keystr(cfg._make_key(section_name, key.strip()))
@@ -706,10 +731,15 @@ class IniFormat(Format):
             lines.append(comment)
         
         # file has been read. assign the values
-        for key, value in values.items():
-            section = cfg.section(key)
+        for i, (key, value) in enumerate(values.items(), 1):
+            try:
+                section = cfg.section(key)
+            except StrictError as e:
+                self._error(e, file, i, lines[i-1].line.strip())
+                continue
+            
             if not section._dirty and value is not None:
-                section.convert(value)
+                section.convert(value, decode=True)
                 section._dirty = False
             if key in comments:
                 section.comment = comments[key]
@@ -745,13 +775,9 @@ class IniFormat(Format):
             # value section
             cfg = self.config
             key = cfg._keystr(cfg._make_key(section.key)[1:])
-            value = section.adapt()
             write(key)
             file.write(self.delimeter)
-            if isinstance(value, bytes):
-                file.write(value)
-            else:
-                write(value)
+            file.write(section.adapt(encode=True))
             write('\n')
         
         section._dirty = False
@@ -764,7 +790,7 @@ class IniFormat(Format):
         header = None
         lines = lines or []
         first = True
-        for i, line in enumerate(lines):
+        for i, line in enumerate(lines, 1):
             if line.issection:
                 if line.name in seen:
                     continue
@@ -777,7 +803,12 @@ class IniFormat(Format):
                             seen.add(sec.key)
                 
                 # write current section header
-                header = cfg.section(line.name)
+                try:
+                    header = cfg.section(line.name)
+                except StrictError as e:
+                    self._error(e, file, i, line.line.strip())
+                    continue
+                
                 self.write_section(header, file, first)
                 seen.add(header.key)
                 first = False
@@ -785,7 +816,14 @@ class IniFormat(Format):
             elif line.iskey:
                 if line.name in seen:
                     continue
-                self.write_section(cfg.section(line.name), file)
+                
+                try:
+                    section = cfg.section(line.name)
+                except StrictError as e:
+                    self._error(e, file, i, line.line.strip())
+                    continue
+                
+                self.write_section(section, file)
                 seen.add(line.name)
             
             else:
@@ -811,6 +849,11 @@ if WIN:
     class RegistryFormat(Format):
         name = 'registry'
         base_key = winreg.HKEY_CURRENT_USER
+        types = {
+            str: winreg.REG_SZ,
+            bytes: winreg.REG_BINARY,
+            int: winreg.REG_DWORD,
+            }
         
         def read(self, key, section=None):
             section = section if section is not None else self.config
@@ -820,7 +863,14 @@ if WIN:
             for i in range(n_values):
                 name, value, type = winreg.EnumValue(key, i)
                 subsection = section.section(name)
-                subsection.set_value(value)
+                
+                reg_type = self.types.get(subsection.type)
+                if reg_type is None:
+                    # not a type supported by the registry, so convert it
+                    subsection.convert(value, decode=True)
+                else:
+                    subsection.set_value(value)
+                
                 subsection._dirty = False
             
             # read values from next subkeys
@@ -842,11 +892,18 @@ if WIN:
                     rkey = ntpath.sep.join(section_key[:-1])
                     name = section_key[-1]
                 
+                # get a supported type for the value
+                reg_type = self.types.get(section.type)
+                if reg_type is None:
+                    # not a type supported by the registry, so adapt it
+                    reg_type = winreg.REG_BINARY
+                    value = section.adapt(encode=True)
+                else:
+                    value = section.value()
+                
                 # write the value
                 subkey = winreg.CreateKeyEx(key, rkey)
-                value = section.value()
-                type = self._get_type(value)
-                winreg.SetValueEx(subkey, name, 0, type, value)
+                winreg.SetValueEx(subkey, name, 0, reg_type, value)
         
         def open(self, source, mode='rb'):
             if 'r' in mode:
@@ -884,17 +941,6 @@ if WIN:
                     yield subsubkey
             
             yield key
-        
-        def _get_type(self, value):
-            if isinstance(value, str):
-                return winreg.REG_SZ
-            elif isinstance(value, bytes):
-                return winreg.REG_BINARY
-            elif isinstance(value, int):
-                return winreg.REG_DWORD
-            else:
-                err = 'type not supported by this format: {}'
-                raise ValueError(err.format(_type(value)))
 
 ## Config Errors ##
 
@@ -910,42 +956,11 @@ class InvalidSectionError(KeyError, ConfigError):
 class NoValueError(ValueError, ConfigError):
     """Raised when a section has no set value or default value."""
 
-class SyncError(ConfigError):
-    """Base class for errors that can occur when syncing."""
-    def __init__(self, filename=None, message=''):
-        self.filename = filename
-        self.message = message
-    
-    def __str__(self):
-        if self.filename:
-            err = ["error reading '{}'".format(self.filename)]
-            if self.message:
-                err.extend([': ', self.message])
-            return ''.join(err)
-        else:
-            return self.message
+class StrictError(ConfigError):
+    """Raised when "strict" mode is enabled and an action is not permitted."""
 
-class ReadError(SyncError):
-    """Raised when a value could not be read from a source."""
-    def __init__(self, filename=None, lineno=None, text='', message=''):
-        super(ReadError, self).__init__(filename, message)
-        self.lineno = lineno
-        self.text = text
-    
-    def __str__(self):
-        if self.filename:
-            msg = "error reading '{}', line {}"
-            err = [msg.format(self.filename, self.lineno)]
-            if self.message:
-                err.extend([': ', self.message])
-            if self.text:
-                err.extend(['\n  ', self.text])
-            return ''.join(err)
-        else:
-            return self.message
-
-class WriteError(SyncError):
-    """Raised when a value could not be written to a source."""
+class FormatError(ConfigError):
+    """Raised for errors when reading/writing with a Format."""
 
 class NoSourcesError(ConfigError):
     """Raised when there are no sources for a config object."""
@@ -957,6 +972,11 @@ class NoValue:
         return '{}'.format(self.__class__.__name__)
 NoValue = NoValue()
 
+def kwargs_check(name, kwargs):
+    if kwargs:
+        err = "{}() got an unexpected keyword argument '{}'"
+        raise TypeError(err.format(name, kwargs.popitem()[0]))
+
 # adapted from pyglet
 def get_source(filename, scope='script'):
     """Returns a path for *filename* in the given *scope*.
