@@ -44,17 +44,17 @@ class TestBasic(unittest.TestCase):
         self.assertIn('ini', profig.Config.known_formats())
         
         c = profig.Config()
-        self.assertIsInstance(c._format, profig.IniFormat)
+        self.assertIsInstance(c.format, profig.IniFormat)
         
         c = profig.Config(format='ini')
-        self.assertIsInstance(c._format, profig.IniFormat)
+        self.assertIsInstance(c.format, profig.IniFormat)
         
         c = profig.Config(format=profig.IniFormat)
-        self.assertIsInstance(c._format, profig.IniFormat)
+        self.assertIsInstance(c.format, profig.IniFormat)
         
         c = profig.Config()
         c.set_format(profig.IniFormat(c))
-        self.assertIsInstance(c._format, profig.IniFormat)
+        self.assertIsInstance(c.format, profig.IniFormat)
         
         with self.assertRaises(profig.UnknownFormatError):
             c = profig.Config(format='marshmallow')
@@ -224,14 +224,13 @@ class TestBasic(unittest.TestCase):
         c['a'] = 1
         self.assertEqual(c.as_dict(), {'a': 1})
         
+        c['c.a'] = 1
+        self.assertEqual(c.as_dict(), {'a': 1, 'c': {'a': 1}})
+        
         c['b'] = 1
         c['a.a'] = 1
-        self.assertEqual(c.as_dict(), {'a': {'': 1, 'a': 1}, 'b': 1})
-        #self.assertEqual(c.as_dict(convert=False),
-            #{'a': {'': '1', 'a': '1'}, 'b': '1'})
-        self.assertEqual(c.as_dict(flat=True), {'a': 1, 'a.a': 1, 'b': 1})
-        #self.assertEqual(c.as_dict(flat=True, convert=False),
-            #{'a': '1', 'a.a': '1', 'b': '1'})
+        self.assertEqual(c.as_dict(), {'a': {'': 1, 'a': 1}, 'b': 1, 'c': {'a': 1}})
+        self.assertEqual(c.as_dict(flat=True), {'a': 1, 'a.a': 1, 'b': 1, 'c.a': 1})
     
     def test_reset(self):
         c = profig.Config(dict_type=dict)
@@ -254,6 +253,44 @@ class TestBasic(unittest.TestCase):
         c.reset()
         self.assertEqual(c.as_dict(flat=True), {'a': 1, 'a.a': 1})
 
+class TestStrictMode(unittest.TestCase):
+    def setUp(self):
+        self.c = profig.Config(strict=True)
+        self.c.init('a', 1)
+    
+    def test_set_init(self):
+        self.c['a'] = 3
+        self.assertEqual(self.c['a'], 3)
+    
+    def test_set_uninit(self):
+        with self.assertRaises(profig.StrictError):
+            self.c['b'] = 3
+        
+        s = self.c.section('b')
+        with self.assertRaises(profig.StrictError):
+            s.set_value(3)
+    
+    def test_read_uninit(self):
+        buf = io.BytesIO(b"""\
+[a]
+a = 1
+""")
+        self.c.format.error_mode = 'exception'
+        with self.assertRaises(profig.StrictError):
+            self.c.read(buf)
+    
+    def test_clear_uninit_on_sync(self):
+        buf = io.BytesIO(b"""\
+[a]
+a = 1
+""")
+        
+        self.c.format.error_mode = 'ignore'
+        self.c.sync(buf)
+        self.assertEqual(buf.getvalue(), b"""\
+[a] = 1
+""")
+
 class TestIniFormat(unittest.TestCase):
     def setUp(self):
         self.c = profig.Config(format='ini')
@@ -532,12 +569,12 @@ class TestCoercer(unittest.TestCase):
             c.read(buf)
 
 class TestErrors(unittest.TestCase):
-    def test_ReadError(self):
+    def test_FormatError(self):
         c = profig.Config()
-        c._format.read_errors = 'exception'
+        c.format.error_mode = 'exception'
         
         buf = io.BytesIO(b"""a""")
-        with self.assertRaises(profig.ReadError):
+        with self.assertRaises(profig.FormatError):
             c.sync(buf)
 
 class TestMisc(unittest.TestCase):
@@ -559,7 +596,7 @@ if profig.WIN:
             self.c = profig.Config(self.path, format='registry')
         
         def tearDown(self):
-            self.c._format.delete(self.key)
+            self.c.format.delete(self.key)
     
         def test_basic(self):
             c = self.c
@@ -642,11 +679,33 @@ if profig.WIN:
         
         def test_unicode_write(self):
             c = self.c
-            c['\uff9c'] = '\uff9c'
+            c.init('\uff9c', '\uff9c')
             c.write()
             
             value = winreg.QueryValueEx(self.key, '\uff9c')[0]
             self.assertEqual(value, '\uff9c')
+        
+        def test_unsupported_type_read(self):
+            key = winreg.CreateKeyEx(self.key, 'a')
+            winreg.SetValueEx(key, '', 0, winreg.REG_SZ, '1.11')
+            
+            c = self.c
+            c.init('a', 1.11)
+            c.read()
+            
+            self.assertEqual(c['a'], 1.11)
+        
+        def test_unsupported_type_write(self):
+            c = self.c
+            c.init('a', 1.11)
+            c.write()
+            
+            value = winreg.QueryValueEx(self.key, 'a')[0]
+            self.assertEqual(value, b'1.11')
 
 if __name__ == '__main__':
+    # silence logging
+    import logging
+    logging.basicConfig(level=logging.CRITICAL)
+    
     unittest.main()
